#include <stdio.h>
#include <stream.h>
#include "std.h"
#include "colony.h"
#include <stdlib.h>

#include <fstream.h>

colony::colony()
{
    total_size = 1.0 ;
    n = default_size ;
    make_room() ;
}


colony::colony( int size )
{
    total_size = 1.0 ;
    n = size ;
    make_room() ;
}

void 
colony::make_room()
{
    B = new real[n * n * n + n*n + 3 * n] ;
    
    if( B != NULL ) { 
	A = B + n * n * n ;
	p = A + n * n ;
	dp = p + n ;
	s_dp = dp +n ;
    }
}

colony::colony( char *fname)
{
  read_from( fname ) ;
}

void 
colony::read_from( char *fname  )
{
  ifstream *f ;
  char c ;
  f = new ifstream( fname ) ;

  total_size = 1.0 ;
  
  if( f != NULL ) {
    f->get(c) ;
    f->putback(c) ;

    if( c >= 'a' && c <= 'z' ) {
      modern_read( *f ) ;
      delete f ;
    } else {
      delete f ;
      FILE * old_f ;
      old_f = fopen( fname, "r" ) ;
      old_read( old_f ) ;
      fclose( old_f ) ;
    }
  }
}

void
colony::modern_read( istream &f)
{
  char c ;
  f >> c ;

  if( c == 'a' ) {

    read_cleanup(f) ;
    
    char com[255] ;
    
    while( !f.eof() ) {
      f >> com ;
      if( !read_interpret( com, f ) ) 
	break ;
    }
  }
}


void 
colony::read_cleanup(istream &f)
{
  f >> n ;
  f >> env_n ;

  make_room() ;
  int i ;
  // cleaning B
  for( i=0; i< n*n*n ; i++) 
    B[i] = 0.0 ;
  
  //  cleaning A 
  for( i=0; i< n*n ; i++)
    A[i] = 0.0 ;
  
}  
	
int
colony::read_interpret( char *com, istream &f)
{
  int i, j, k ;
  if( strcmp( com, "trans" ) == 0 ) {
    f >> i >> j ;
    f >> c_a(i,j) ;
    return true ;
  } else if( strcmp( com, "meet" ) == 0 ) {
    f >> i >> j >> k ;
    f >> c_b(i,j,k) ;
    return true ;
  } 
  return false ;
}
    

void
colony::old_read( FILE *f)
{
    
    fscanf( f, "%d", &n ) ;
    
    make_room()  ;
    

    int i ;
    for( i=0; i< n*n*n ; i++)
	fscanf( f, "%lg", &(B[i]) ) ;
    
    
    for( i=0; i< n*n ; i++)
	fscanf( f, "%lg", &(A[i]) ) ;
}

int
colony::read_population( char *fname )
{
    FILE *f ;

    f = fopen( fname, "r" ) ;
    
    if( f != NULL ) {
	int n_p,i ;
	fscanf( f, "%d", &n_p ) ;
	if( n_p != n) {
	    fprintf(stderr, "WARNING: population read has different number of states than population in colony.\n") ;
	}
	if( n_p > n)
	    n_p = n ;
	for( i=0; i<n_p; i++)
	    fscanf( f, "%lg", &(p[i]) ) ;
	for( ; i<n; i++)
	    p[i] = 0.0 ;
	fclose( f ) ;
	return true ;
    }
    return false ;
}

colony::colony( colony &oc)
{
    total_size = 1.0 ;
    n = oc.n ;
    make_room() ;
    int i,e ;

    e = n * n * n + n*n + 2 * n ;
    for( p = B,i=0; i< e; i++)
	B[i] = oc.B[i] ;
}

colony::~colony()
{
    delete B;
}

colony &colony::operator=(colony &oc)
{
    if( n != oc.n ) {
	if( n != 0 )
	    delete B ;
	n = oc.n ;
	make_room() ;
    }

    int i,e ;

    e = n * n * n + n*n + 2 * n ;
    for( i=0; i< e; i++)
	B[i] = oc.B[i] ;
    total_size = oc.total_size ;

    return *this ;
}    


int 
colony::write( char *fname  )
{
    FILE *f ;
    int i,j,k ;

    real *i_p ;

    f = fopen( fname, "w" ) ;

    if( f == NULL )
	return false ;

    fprintf( f, "%d\n", n ) ;
  
    for( i=0, i_p = B ; i< n ; i++) {
	for( j=0; j < n; j++) {
	    for( k=0; k<n; k++, i_p++ ) {
		fprintf( f, "%g ", *i_p ) ;
	    }
	    fprintf( f, "\n" ) ;
	}
	fprintf( f, "\n" ) ;
    }

    fprintf( f, "\n" ) ;

    for( i=0, i_p = A ; i< n ; i++) {
	for( k=0; k<n; k++, i_p++ ) {
	    fprintf( f, "%g ", *i_p ) ;
	}
	fprintf( f, "\n" ) ;
    }
    fclose( f ) ;
    return true ;
}

real &
colony::c_a( int i, int j)
{
    return A[i*n+j]; 
}

real &
colony::c_b( int i, int j, int k)
{
    return B[i*n*n+j*n+k] ;
}

real 
colony::dp_dt( int i)
{
    int j,k ;
    real der ;

    for( j=0, der = 0.0 ; j<n; j++ ) {
	der += -c_a(i,j) * p[i] + c_a(j,i) * p[j] ;
    }

    for( j=0; j<n; j++ ) 
	for( k=0; k<n; k++ ) {
	    der += -c_b(i,j,k) * p[i] * p[j] + c_b(j,k,i) * p[j] * p[k] ;
	}

    return der ;

}

void 
colony::compute_dp()
{
    int i ;
    for( i=0; i< n; i++ )
	dp[i] = dp_dt( i ) ;
}

void colony::advance( real dt)
{
    int i ;
    for( i=0; i< n; i++ )
	p[i] += dp[i] * dt ;
}

void 
colony::transform( real dt )
{
    compute_dp( ) ;

    advance( dt ) ;
}

void 
colony::printf_population()
{
    int i ;
  
    for( i=0; i<n; i++)
	printf( "%15.13g ", p[i] ) ;
}

void 
colony::random_ants( )
{
    int i ;
    real s;
    for( i=0, s=0.0; i< n; i++) {
	p[i] = random_real() ;
	s += p[i] ;
    }

    s = 1 / s ;
  
    for( i=0; i< n ; i++ ) 
	p[i] *= s ;
}

void 
colony::uniform_population()
{
    int i ;
    real u_p = total_size / n ;
    
    for( i=0; i<n; i++)
	p[i] = u_p ;
}

void 
colony::random_weights()
{
    int i ;
    for( i=0; i<n*n*n; i++)
	B[i] = random_real()  ;

    for( i=0; i < n*n; i++)
	A[i] = 0.1 * random_real()  ;
}

real &
colony::c_p(int i)
{
    return p[i] ;
}

real &
colony::c_dp(int i)
{
    return dp[i] ;
}

int 
colony::size()
{
    return n ;
}

void 
colony::set_total( real s )
{
    total_size = s ;
}

void
colony::save_dp()
{
    int i ;
    for( i=0; i<n; i++)
	s_dp[i] = dp[i] ;
}


void
colony::retr_dp()
{
    int i ;
    for( i=0; i<n; i++)
	dp[i] = s_dp[i] ;
}







