#include "disc_colony.h"

#include <string.h>
#include <fstream.h>

#include <ncurses.h>

void 
discrete_colony::make_more_room()
{
  no_c_B = new real[ n*n +n] ;
    
  //    if( B != NULL ) { 
  c_a_multi = new multi_nomial[ n * env_n ] ;
  A = new real[env_n * n * n] ;
  {int i; for(i=0; i<n*n*env_n; i++) A[i]=0.00;}
  //   }
  dir_multi = new multi_nomial[ n * env_n ] ;
  ants =new int[size_x * size_y * 2] ;
  n_filled = new int[size_x * size_y ] ;
  envir = new int[size_x * size_y] ;
}

void 
discrete_colony::make_room_for_env()
{
  delete [] ants ;
  delete [] n_filled ;
  delete [] envir ;
  ants = new int[ size_x * size_y * 2 ] ;
  n_filled = new int[size_x * size_y ] ;
  envir = new int[size_x * size_y ] ;
}

    

// remove an ant p from place x,y
// this routine will work even if more than 2 ants
// per place are allowed
void 
discrete_colony::remove_ant( int ant_num )
{
  int x = ant_record[ ant_num].x ;
  int y = ant_record[ ant_num].y ;
  int p = ant_record[ ant_num ].p ;
  //  int st = ant_record[ ant_num].sta ;
  num_ants( x, y)-- ;
  if( (num_ants( x,y) > 0) && (p == 0)) {                //still ants left...
    ant( x, y, 0 ) = ant( x, y, num_ants(x,y) ) ; // take last
    ant_record[ ant(x,y,0 ) ].p = 0 ;
  }
  //  c_p( st )-- ;
}


void 
discrete_colony::add_ant( int ant_num, int x, int y, int sta )
{
  ant_record[ ant_num].x = x ;
  ant_record[ ant_num].y = y ;
  ant_record[ ant_num].sta = sta ;
  ant_record[ ant_num].dir = 0 ;

  ant( x, y, num_ants( x, y ) ) = ant_num ;
  ant_record[ ant_num ].p = num_ants(x,y) ;
  num_ants( x, y)++ ;
  //    c_p( sta )++ ;
}
    
int
discrete_colony::choose_dir( int sta, int x, int y ) 
{
  return dir_multi_a( sta, env(x,y)).choose() ;
}

const int x_dir[] = { -1, 0, 1, 
		      -1, 0, 1,
		      -1, 0, 1 } ;
const int y_dir[] = { -1, -1, -1,
		      0,   0,  0, 
		      1,   1,  1 } ;

int 
discrete_colony::test_ps()
{
  int s[20],x,y ;
  for( x=0; x<20; x++) s[x] = 0 ;
  for( y=0; y<size_y; y++)
    for( x=0; x<size_x; x++) 
      switch ( num_ants(x,y) ) {
      case 0:
	break ;
      case 1:
	s[ant_record[ant(x,y,0)].sta]++ ;
	break ;
      default:
	s[ant_record[ant(x,y,0)].sta]++ ;
	s[ant_record[ant(x,y,1)].sta]++ ;
	break ;
      }
  for( x=0; x<n; x++) 
    if( s[x] != c_p(x) )
      return FALSE ;
  for( x=0; x<n_ants;x++) {
    if( ((num_ants( ant_record[ x].x, ant_record[x].y ) == 1) &&
	 (ant_record[ x]).p == 1 )
	 ||
	 (num_ants( ant_record[ x].x, ant_record[x].y ) == 0) )
				     
      return FALSE ;
  }
  return TRUE ;
}

int
discrete_colony::go( int an)
{
  int x, y, p ;
  x = ant_record[an].x ;
  y = ant_record[an].y ;
  p = ant_record[an].p ;
  int old_st = ant_record[ant( x, y, p )].sta ;
  int direction = dir_multi_a( old_st, env(x, y)).choose() ;
  int nx = x_dir[direction] + x ;
  int ny = y_dir[direction] + y;

  if( !place_free( nx, ny ) ) {
    nx = x ;
    ny = y ;
  } 
  if( !test_ps() )
    cout << "somethings fishy...before\n" ;
  {

    remove_ant( an ) ;
    if( num_ants( nx, ny ) == 1 ) 
      old_st = meeting( ant_record[ant(nx, ny, 0)].sta, old_st ) ;
    int new_st = ca_multi_env( old_st, nx, ny ) ;
    if( old_st != new_st ) {
      c_p( old_st )-- ;
      c_p( new_st )++ ;
    }
    add_ant( an, nx, ny, new_st ) ;
    ant_record[an ].dir = direction ;
  }
  return TRUE ;
}
   
    
void
discrete_colony::transform(real) 
{
  int i ;

  for( i=0; i< n_ants; i++)
    go( i ) ;

}

void 
discrete_colony::choose_state( real no_change, real *c, int &st )
{
    
  real r = random_real() - no_change ;
  if( r > 0 ) {
    int new_st ;
    for( new_st = 0; r > 0 && new_st<n; r-= c[new_st], new_st++ )  ;
    new_st-- ;
    c_p( st ) -- ;
    c_p( new_st ) ++ ;
    st = new_st ;
  }
}

int 
discrete_colony::meeting( int st_a, int st_b )
{
  choose_state( c_nB( st_b, st_a ), &(c_b( st_b, st_a, 0)), st_b ) ;
  return st_b ;
}

int &
discrete_colony::ant( int x, int y, int pl)
{
  return ants[  (y * size_x + x) * 2 + pl  ] ;
}

int &
discrete_colony::num_ants( int x, int y)
{
  return n_filled[ y * size_x + x ] ;
}


int
discrete_colony::place_free( int x, int y)
{
  return (env(x,y) > 0) && (num_ants( x, y ) < 2) ;
}

real &
discrete_colony::c_nB( int i, int j)
{
  return no_c_B[ i * n + j ] ;
}

discrete_colony::discrete_colony( 
				 char *fname, 
				 int total_ants, 
				 real a, int x, int y):
  colony(),
  no_c_B( NULL ),
  c_a_multi( NULL ),
  dir_multi( NULL )
{
  ifstream f ;
  size_x = x ;
  size_y = y ;
  alfa_n = a ;

  ant_record = NULL ;
  n_ants = total_ants ;

  read_from( fname ) ;
  //  f.open( fname ) ;
  //  modern_read( f ) ;
  //  f.close() ;

  int i, *ip ;
  for( i=0, ip = n_filled; i<size_x*size_y; i++, ip++) 
    *ip = 0 ;

  setup_c_a_multi() ;
  setup_c_nB() ;
}

void 
discrete_colony::setup_c_a_multi()
{
  int e,i,j ;
  real s ;
  for( e=0; e<env_n; e++)
    for( i=0; i<n; i++) {
      c_a_multi_a(i,e).reset() ;
      for( j=0, s=1.0; j<n; j++) {
	s -= c_a( i,j, e ) ;
	if( c_a(i,j,e) > 0 ) {
	  c_a_multi_a(i,e).add( c_a( i, j, e ) , j) ;
	}
      }
      c_a_multi_a(i,e).add( s, i ) ;
    }
}


void 
discrete_colony::setup_c_nB()
{
  int i,j,k ;
  real s ;
  for( i=0; i<n; i++) 
    for( j=0; j<n; j++) {
      for( k=0, s=1.0; k<n; k++)
	s -= c_b( i,j, k) ;
      c_nB( i,j ) = s ;
    }
	
}


void 
discrete_colony::test_trans()
{
  int i, j, e ;
  for( e=0; e<env_n; e++)
    for( i=0; i<n; i++) {
      cout << "<" << i << "," << e << ">" ;
      for( j=0; j<10; j++)
	cout << c_a_multi_a(i,e).choose()  ;
    }
}

void
discrete_colony::distribute_ants()
{
  int nx, ny ;
  int i,j, *ip ;
  for( i=0, ip = n_filled; i<size_x*size_y; i++, ip++) 
    *ip = 0 ;

  ant_record = new ant_dat[n_ants] ;
    
  int ant_counter = 0 ;
  for( i=0; i<n; i++) 
    for( j=0; j<p[i]; j++) {
      do {
	nx = random_long( size_x ) ;
	ny = random_long( size_y ) ; 
      } while( !place_free( nx, ny )  ) ;
      add_ant( ant_counter, nx, ny, i )  ;
      ant_counter++ ;
    }
}

void 
discrete_colony::choose_colony_states()
{
  int nst ;
  int i ;

  multi_nomial mc ;
 
  for( i=0; i<n; i++) {
    mc.add( p[i], i ) ;
    p[i] = 0 ;
  }

  for( i=0; i<n_ants; i++) {
    nst = mc.choose() ;
    p[nst]++ ;
  }
}
discrete_colony::~discrete_colony()
{
  delete[] no_c_B ;
  delete[] c_a_multi ;
  delete[] dir_multi ;
  delete[] ants ;
  delete[] n_filled ;
}

void
discrete_colony::uniform_population()
{
  colony::uniform_population() ;
  for( int i=0; i<n; i++)
    c_p(i) *= n_ants ;
  choose_colony_states() ;
  distribute_ants() ;
}


char* 
discrete_colony::env_states = "0123456789abcdefghijklmnopqrstuvwxyz" ;


void 
discrete_colony::read_cleanup(istream &f) 
{
  colony::read_cleanup(f) ;
  make_more_room() ;
  int i, j, sta ;
  real chan ; 
  for( sta=0; sta<n; sta++) {
    for( j=0; j<env_n; j++)
      dir_multi_a(sta,j).reset() ;
    for( i=0; i<9; i++ ) {
      if( i==4 ) chan=0 ;
      else chan=1 ;
      for( j=0; j< env_n; j++)
	if( chan > 0 )
	  dir_multi_a(sta, j).add( chan, i ) ;
    }
  }
}

int
discrete_colony::read_interpret( char *com, istream  &f)
{
  if( strcmp( com, "env" ) == 0) {
    f >> size_y >> size_x ;
    make_room_for_env() ;
    int i, j, ci ;
    char c ;
    for( i=0; i<size_x; i++) {
      f >> c ; 
      ci = c ;
      env( i, 0 ) = strchr( env_states, ci ) - env_states ;
      for( j=1; j<size_y; j++) {
	f >> c ;
	ci = c ;
	env( i, j ) = strchr( env_states, ci ) - env_states ;
      }
    }
    return TRUE ;
  } else if( strcmp( com, "etrans" ) == 0 ) {
    int i, j, esta ;
    char cenv ;
    real chan ;
    f >> cenv >> i >> j >> chan ;

    esta = strchr( env_states, cenv ) - env_states ;
    c_a( i, j, esta ) = chan ;
    return TRUE ;
  } else if( strcmp( com, "trans" ) == 0 ) {
    int i, j, esta ;
    real chan ;
    f >> i >> j >> chan ;

    for( esta = 0; esta < env_n; esta++)
      c_a( i, j, esta ) = chan ;
    return TRUE ;
  } else if( strcmp( com, "edir" ) == 0 ) {
    int sta, esta ;
    char cenv ;
    f >> cenv >> sta ;
    esta = strchr( env_states, cenv ) - env_states ;
    dir_multi_a(sta,esta).reset() ;
    int i ; real chan ;
    for( i=0; i<9; i++ ) {
      f >> chan ;
      dir_multi_a(sta,esta).add( chan, i ) ;
    }
    return TRUE ;
  } else if( strcmp( com, "dir" )  == 0) {
    int sta ;
    f >> sta ;
    int i, j ;
    real chan ; 
    for( j=0; j<env_n; j++)
      dir_multi_a(sta,j).reset() ;
    for( i=0; i<9; i++ ) {
      f >> chan ;
      if( chan > 0 )
	for( j=0; j< env_n; j++)
	  dir_multi_a(sta, j).add( chan, i ) ;
    }
    return TRUE ;
  } else 
    return colony::read_interpret( com, f ) ;
}



int
discrete_colony::read_population( char *fname )
{
  int ret = colony::read_population( fname ) ;
  if( ret ) {
    real sum = 0.0 ;
    for( int i=0; i<n; i++) 
      sum += c_p(i) ;
    if( sum > 0 ) sum = 1/sum ;
    if( sum < 0.5 ) {
      for( int i=0; i<n; i++)
	c_p(i) *= sum * n_ants ;
      choose_colony_states() ;
    } else {
      for( int i=0; i<n; i++)
	c_p(i) *= sum * n_ants ;
    }
    distribute_ants() ;
  }
  return ret ;
}

real &
discrete_colony::c_a( int i, int j, int st )
{
  return A[ st * n * n + i * n + j ] ;
}

int 
discrete_colony::ca_multi_env( int sta, int x, int y )
{
  return c_a_multi_a(sta,env(x,y)).choose() ;
}

int &
discrete_colony::env( int x, int y)
{
  return envir[  (y * size_x + x)] ;
}


multi_nomial &
discrete_colony::dir_multi_a( int x_sta, int y_esta )
{
  return dir_multi[ y_esta * n + x_sta ] ;
}

multi_nomial &
discrete_colony::c_a_multi_a( int x_sta, int y_env )
{
  return c_a_multi[ y_env * n + x_sta ] ;
}
       




