#include "gled_disc.h"
#include "fstream.h"

extern "C" {
#include <GL/gl.h>
#include <GL/glu.h>
#include <stdlib.h>
}

#define WITH_ENV

GLuint antName = 1;
GLuint envName = 2 ;

  


double ant_color[] = {
  1.0, 0.0, 0.0 ,
  0.0, 1.0, 0.0 ,
  1.0, 1.0, 0.0 ,
  1.0, 1.0, 1.0 , 
  1.0, 0.0, 1.0,
  0.0, 1.0, 1.0,
  0.7, 0.7, 0.7,
  0.4, 0.7, 1.0
} ;

double env_color[] = {
  1.0, 0.0, 0.0 ,
  0.0, 1.0, 0.0 ,
  1.0, 1.0, 0.0 ,
  1.0, 1.0, 1.0 , 
  1.0, 0.0, 1.0,
  0.0, 1.0, 1.0,
  0.7, 0.7, 0.7,
  0.4, 0.7, 1.0
} ;

void put_ant( double x, double y, double angle, int color )
{
    glPushMatrix() ;
    color=(color%8)*3 ;
    glColor3f (ant_color[color], ant_color[color+1], ant_color[color+2]);
    glTranslatef (x*2-0.5, y*2-1, 0.0);
    glRotatef( angle * 45, 0, 0, 1 ) ;
    glCallList( antName) ;
    glPopMatrix() ;
}



gled_colony::gled_colony( char *fname, int total_ants, real a, int x, int y) :
    discrete_colony(
	fname, 
	total_ants, 
	a, 
	x, y)
{
  gluOrtho2D(-1,-1,size_y*2+1,size_x*2+1) ;
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  
  initAnt() ;
  initEnv() ;
}

void gled_colony::initAnt()
{
    glNewList (antName, GL_COMPILE);
	glBegin (GL_LINES);
	glVertex2f (0.0, 0.0);
	glVertex2f (0.8, 0.6);

	glVertex2f (0.0, 0.0);
	glVertex2f (0.1, 0.85);

	glVertex2f (0.0, 0.0);
	glVertex2f (-0.8, 0.6);

	glVertex2f (0.0, 0.0);
	glVertex2f (0.7, -0.6);

	glVertex2f (0.0, 0.0);
	glVertex2f (-0.2, -0.85);

	glVertex2f (0.0, 0.0);
	glVertex2f (-0.8, -0.6);

	glVertex2f (0.8, 0.0);
	glVertex2f (1.0, 0.5);

	glVertex2f (0.8, 0.0);
	glVertex2f (1.0, -0.5);
	glEnd ();

        glBegin( GL_TRIANGLES) ;
        glVertex2f (0.5,0.0) ;
	glVertex2f (0.8, 0.3);
	glVertex2f (0.8, -0.3);

        glVertex2f (0.5,0.0) ;
	glVertex2f (0.0, 0.2);
	glVertex2f (-0.4, 0.0);

        glVertex2f (0.5,0.0) ;
	glVertex2f (0.0, -0.2);
	glVertex2f (-0.4, 0.0);

        glVertex2f (-0.3,0.0) ;
	glVertex2f (-0.8, 0.3);
	glVertex2f (-1.1, 0.0);

        glVertex2f (-0.3,0.0) ;
	glVertex2f (-0.8, -0.3);
	glVertex2f (-1.1, 0.0);
        glEnd() ;
     glEndList ();
}


int
gled_colony::read_interpret( char *com, istream  &f)
{
  if( strcmp( com, "color" ) == 0 ) {
    int sta ;
    real red, green , blue ;
    f >> sta >> red >> green >> blue;
    sta = (sta % 8)*3 ;
    ant_color[sta] = red ; ant_color[sta+1]=green; ant_color[sta+2]=blue ;
    return TRUE ;
  } else 
    return discrete_colony::read_interpret( com, f ) ;
}


void gled_colony::initEnv() 
{
#ifdef WITH_ENV
  double x,y ;
  int color ;
  glNewList( envName, GL_COMPILE) ;
  glBegin (GL_LINES) ;
  for( y=0; y<size_x; y++)
    for( x=0; x<size_y; x++) {
      if( env((int)y,(int)x) != 1 ) {
	color = (env((int)y,(int)x)%8)*3 ;
	glColor3f (env_color[color], env_color[color+1], env_color[color+2]);
	glVertex2f( x*2+0.4,y*2 ) ;
	glVertex2f( x*2+2+0.4,y*2+2) ;

	glVertex2f( x*2+2.4,y*2 ) ;
	glVertex2f( x*2+0.4,y*2+2) ;
      }
    }
  glEnd() ;
  glEndList ();
#endif
}

gled_colony::~gled_colony()
{
}

int dirtrans[] = { 5,4,3,
                   6,0,2,
                   7,0,1,1,1 } ;
int s_record[200][20] ;
int cur_s=0 ;



void
gled_colony::draw()
{
    int x, y ;
#ifdef WITH_ENV
    glCallList( envName) ;
#endif
    cur_s = (cur_s+1)%100;
    for( x=0; x<20; x++) s_record[cur_s][x] = 0 ;
    for( y=0; y<size_y; y++)
      for( x=0; x<size_x; x++) {
	    switch ( num_ants(x,y) ) {
	    case 0:
	      break ;
	    case 1:
	      put_ant( 1.0+y,1.0+x,dirtrans[ant_record[ant(x,y,0)].dir], ant_record[ant(x,y,0)].sta ) ;
	      s_record[cur_s][ant_record[ant(x,y,0)].sta]++ ;
	      break ;
	    default:
	      put_ant( 1.35+y,1.35+x,5, ant_record[ant(x,y,0)].sta ) ;
	      put_ant( 0.65+y,0.65+x,1, ant_record[ant(x,y,1)].sta ) ;
	      s_record[cur_s][ant_record[ant(x,y,0)].sta]++ ;
	      s_record[cur_s][ant_record[ant(x,y,1)].sta]++ ;
	      break ;
	    }
      }
#define drawOneLine(x1,y1,x2,y2) glBegin(GL_LINES); \
        glVertex2f ((x1),(y1)); glVertex2f ((x2),(y2)); glEnd();

{    glPushMatrix() ;
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(-5.2,1,100,0) ;
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
     glColor3f(1.0,1.0,1.0) ; 
    {
      int i,j ;
      cout << cur_s ;
      for( j=0; j<8; j++)
	for( i=0; i<99; i++) {
	  glColor3f (ant_color[(j%8)*3], ant_color[(j%8)*3+1], ant_color[(j%8)*3+2]);
	  drawOneLine(1.0/n_ants*s_record[(i+cur_s)%100][j],i,
		      1.0/n_ants*s_record[(i+cur_s+1)%100][j],i+1) ;
      }
    }

    glPopMatrix() ;
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0,2.4*size_y,2*size_x,0) ;
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    }
}


 



