#ifndef _MERSENNE_H
#define _MERSENNE_H

typedef unsigned long uint32;

#define MERSENNE_N              (624)                 // length of state vector



#define WITH_SWAP 1

typedef struct _mersenne_data {
  int left ;
  uint32 *next ;
  uint32 state[MERSENNE_N+1] ;
} mersenne_data;


#ifndef _MERSENNE_C
extern mersenne_data *mersenne ;
#else
mersenne_data mersenne_buffer = {
  -1, NULL, {}
} ;
mersenne_data *mersenne = &mersenne_buffer ;
#endif


uint32 reloadMT(void);
void seedMT( uint32) ;

#if WITH_SWAP
uint32 reloadMT_b(mersenne_data *);
void seedMT_b( mersenne_data *,uint32) ;
mersenne_data *swap_with_mersenne_buffer( mersenne_data *buf);


inline uint32 randomMT_b(mersenne_data *buf)
{
  uint32 y;
  
  if(--buf->left < 0)
    return(reloadMT_b(buf));
  
  y  = *(buf->next)++;
  y ^= (y >> 11);
  y ^= (y <<  7) & 0x9D2C5680U;
  y ^= (y << 15) & 0xEFC60000U;
  return(y ^ (y >> 18));
}

#endif
inline uint32 randomMT(void)
{
  uint32 y;
  
  if(--mersenne->left < 0)
    return(reloadMT());
  
  y  = *(mersenne->next)++;
  y ^= (y >> 11);
  y ^= (y <<  7) & 0x9D2C5680U;
  y ^= (y << 15) & 0xEFC60000U;
  return(y ^ (y >> 18));
}


#endif // _MERSENNE_H
