#ifndef _STD_H
#define _STD_H

#ifndef JUST_RAND
#ifndef RAND48
#ifndef RANLIB
#ifndef MERSENNE_RAND


//#error Please define JUST_RAND, RAND48, RANLIB, or MERSENNE_RAND
//#ifdef _MICROSOFT_
//#define JUST_RAND
//#else
//#define MERSENNE_RAND
//#endif
#endif // MERSENNE_RAND
#endif // RANLIB
#endif // RAND48
#endif // JUST_RAND

#ifdef MERSENNE_RAND
extern "C" {
#include <mersenne.h>
}
#endif

#ifdef RAND48
#include <stdlib.h>
#endif

#ifdef JUST_RAND
#include <stdlib.h>
#endif

#ifdef NO_BOOL
const int FALSE =  (1 != 1) ;
const int TRUE  = ( 1 == 1) ;
#endif

typedef double real ; 
typedef long integer ;

template<class T> 
	class listable {
public:
	virtual bool operator==(const listable &g_o ) const {
		return this == &g_o ;
	}
	virtual bool operator!=(const listable &g_o ) const {
		return this != &g_o ;
	}
	virtual bool operator<(const listable & ) const {
		return false ;
	}
	virtual bool operator>(const listable & ) const {
		return true ;
	}
} ;


#ifdef RANLIB
extern "C" {
#include <ranlib.h>
}

extern "C" {
  float ranf(void);
}

void init_random( long seed )
{
  setall( seed, seed * 42 ) ;
}

real random_real()
{
  return (real) ranf() ;
}

inline 
long random_long()
{
    return ignlgi() ;
}

inline 
long random_long( long n )
{
    return ignuin( 0, n-1 ) ;
}

inline 
int random_int( int n )
{
    return ignuin( 0, n-1 ) ;
}

inline 
int random_int(int s, int e)
{
  return ignuin( s, e ) ;
}
#endif




#ifdef RAND48
inline void init_random( long seed )
{
  srand48( seed ) ;
}


inline real random_real()
{
  return drand48() ;
}

inline long random_long(long n)
{
    return lrand48() % (n) ;
}

inline int random_int(int n)
{
  return lrand48() % (n) ;
}

inline int random_int(int s, int e)
{
  return (lrand48() % (e-s+1))+s ;
}

inline long random_long()
{
    return lrand48() ;
}
#define MAGIC_NUMBER (0xf00d)

class  random_seed_stack {
public:
  int size ;
  int cur ;
  int magic ;
  unsigned short buffer[1000] ;
} ;

class seed_buffer {
public:
  unsigned short buffer[3] ;
} ;

inline void random_push( random_seed_stack &buf, seed_buffer &seed )
{
  unsigned short *temp ;
  if(       (buf.magic  != MAGIC_NUMBER) ) {
    buf.magic = MAGIC_NUMBER ;
    buf.cur = 0 ;
    buf.size = 1000 ;
  } else if( (buf.cur)==
	     (buf.size) ) {
    exit(20) ;
    //    ((random_seed_stack *)buf)->size += 300 ;
    //    buf = (char *)realloc( buf, ((random_seed_stack *)buf)->size * sizeof(unsigned short) + 3*sizeof(int) ) ;
  }
  temp = seed48( seed.buffer ) ;
  buf.buffer[buf.cur++] = temp[0] ;
  buf.buffer[buf.cur++] = temp[1] ;
  buf.buffer[buf.cur++] = temp[2] ;
}

inline void random_pop( random_seed_stack &buf, seed_buffer &seed)
{
  unsigned short *temp ;  
  if( buf.cur > 2 ) {
    buf.cur -= 3 ;
    temp = seed48( (&(buf.buffer[buf.cur] ) ) ) ;
    seed.buffer[0] = temp[0] ;
    seed.buffer[1] = temp[1] ;
    seed.buffer[2] = temp[2] ;
  }
}

inline seed_buffer create_seed( random_seed_stack &buf, long seed )
{
  seed_buffer b ;
  random_push( buf, b ) ;
  init_random( seed ) ;
  random_pop( buf , b) ;
  return b ;
}

inline char *priv_init_random( long seed )
{
  char *buf = (char *)(new long[3]) ;
  ((unsigned short *)buf)[0] = (unsigned short)0x33e16l ;
  ((unsigned short *)buf)[1] = (unsigned short)0x33e16l ;
  ((unsigned short *)buf)[2] = (unsigned short)0x33e16l ;
  *(long *)buf = seed ;
  return buf ;
}


inline real priv_random_real( char *buf)
{
  return erand48((short unsigned int *)buf) ;
}

inline long priv_random_long(char *buf,long n)
{
    return nrand48((short unsigned int *)buf) % (n) ;
}

inline int priv_random_int(char *buf,int n)
{
  return nrand48((short unsigned int *)buf) % (n) ;
}

inline int priv_random_int(char*buf,int s, int e)
{
  return (nrand48((short unsigned int *)buf) % (e-s+1))+s ;
}

inline long priv_random_long(char *buf)
{
    return nrand48((short unsigned int *)buf) ;
}

#endif

#ifdef MERSENNE_RAND
inline void init_random( long seed )
{
  seedMT( seed ) ;
}


inline real random_real()
{
  return (real) randomMT() * 2.3283064365386963e-10  ;
}

inline long random_long(long n)
{
    return randomMT() % (n) ;
}

inline int random_int(int n)
{
  return randomMT() % (n) ;
}

inline int random_int(int s, int e)
{
  return (randomMT() % (e-s+1))+s ;
}

inline long random_long()
{
    return randomMT() ;
}

#define MAGIC_NUMBER (0xf00d)

typedef mersenne_data seed_buffer ;


class  random_seed_stack {
public:
  int size ;
  int cur ;
  int magic ;
  seed_buffer *buffer[1000] ;
} ;


inline void random_push( random_seed_stack &buf, seed_buffer &seed )
{
  if(       (buf.magic  != MAGIC_NUMBER) ) {
    buf.magic = MAGIC_NUMBER ;
    buf.cur = 0 ;
    buf.size = 1000 ;
  } else if( (buf.cur)==
	     (buf.size) ) {
    exit(20) ;
    //    ((random_seed_stack *)buf)->size += 300 ;
    //    buf = (char *)realloc( buf, ((random_seed_stack *)buf)->size * sizeof(unsigned short) + 3*sizeof(int) ) ;
  }
  buf.buffer[buf.cur++] = swap_with_mersenne_buffer( &(seed) ) ;
}

inline void random_pop( random_seed_stack &buf, seed_buffer &)
{
  seed_buffer *temp ;  
  if( buf.cur > 0 ) {
    buf.cur-- ;
    temp = swap_with_mersenne_buffer( (buf.buffer[buf.cur] ) ) ;
  }
}

inline seed_buffer create_seed( random_seed_stack &buf, long seed )
{
  seed_buffer b ;
  random_push( buf, b ) ;
  init_random( seed ) ;
  random_pop( buf , b) ;
  return b ;
}

#if 0
inline char *priv_init_random( long seed )
{
  char *buf = (char *)(new long[3]) ;
  ((unsigned short *)buf)[0] = (unsigned short)0x33e16l ;
  ((unsigned short *)buf)[1] = (unsigned short)0x33e16l ;
  ((unsigned short *)buf)[2] = (unsigned short)0x33e16l ;
  *(long *)buf = seed ;
  return buf ;
}


inline real priv_random_real( char *buf)
{
  return erand48((short unsigned int *)buf) ;
}

inline long priv_random_long(char *buf,long n)
{
    return nrand48((short unsigned int *)buf) % (n) ;
}

inline int priv_random_int(char *buf,int n)
{
  return nrand48((short unsigned int *)buf) % (n) ;
}

inline int priv_random_int(char*buf,int s, int e)
{
  return (nrand48((short unsigned int *)buf) % (e-s+1))+s ;
}

inline long priv_random_long(char *buf)
{
    return nrand48((short unsigned int *)buf) ;
}
#endif

#endif


#ifdef JUST_RAND


#define USE_INT_FOR_RANDOM

inline void init_random( long seed )
{
  srand( seed ) ;
}


inline real random_real()
{
  return 1.0 * rand() / RAND_MAX + rand() / ( 1.0 * RAND_MAX * RAND_MAX ) + rand() / ( 1.0 * RAND_MAX * RAND_MAX * RAND_MAX ) ;
}

inline long random_long(long n)
{
  if( n <= RAND_MAX )
		return rand() ;
	else {
		int i ;
		long r ;
		for( i=0; i< ( sizeof( long ) * 8  ) / (sizeof(int ) * 8 - 1 ); i++ ) {
			r <<= (sizeof( int ) * 8 - 1) ;
			r += rand() ;
		}
		return r % (n);
	}
}

inline int random_int(int n)
{
  return rand() % (n) ;
}

inline int random_int()
{
	return rand() ;
}

inline int random_int(int s, int e)
{
  return (rand() % (e-s+1))+s ;
}

inline long random_long()
{
		int i ;
		long r ;
		for( i=0; i< ( sizeof( long ) * 8  ) / (sizeof(int ) * 8 - 1 ); i++ ) {
			r <<= (sizeof( int ) * 8 - 1) ;
			r += rand() ;
		}
		return r ;
}

#endif

inline real SQR( real x )
{
    return x * x ;
}

inline real CUB( real x )
{
    return x * x * x ;
}


#ifdef RAND_GNU

#include <ACG.h>
#include <Uniform.h>
#include <RndInt.h>

extern ACG *generator ;
inline real random_real()
{
    Uniform uni(0.0, 1.0, generator ) ;
    return uni() ;
}

inline long random_long(int n)
{
    RandomInteger rnd(0,n-1,generator) ;
    return rnd() ;
}
#endif

inline real abs( real x)
{
    return (x > 0) ? x : -x ;
}

inline long max(long a, long b)
{
    if( a > b )
	return a ;
    else
	return b;
}

inline void clip( real &a, real lo, real hi)
{
  if( a <lo ) a = lo ;
  else if( a>hi ) a = hi ;
}

inline void clip( long &a, long lo, long hi)
{
  if( a <lo ) a = lo ;
  else if( a>hi ) a = hi ;
}

inline void clip( int &a, int lo, int hi)
{
  if( a <lo ) a = lo ;
  else if( a>hi ) a = hi ;
}
		

#endif

